﻿/*
www.E2CRE8.be - Brainbox Arduino  - by Bart Huyskens
04/2016 - ATMEL STUDIO 7

This program demonstrates how Atmel Studio 7 can be used to program the Brainbox Arduino - based on the ARduino Leonardo
Look at the manual on www.E2CRE8.be on how to set up STUDIO7 and AVRDUDE to program the Hardware of the BBA

This program generates PWM signals at 6 different output pins - it is necessary to look at the datasheet for further explanation
  // OC1A = PWM at B5
  // OC1B = PWM at B6
  // OC1C = PWM at B7
  // OC3A = PWM at C6
  // OC4A = PWM at C7 - BLUE LED
  // OC4D = PWM at D7

  The PWM duty cycle (0-255) is set by the position of a potmeter connected to  to ADC7 (AN0 ARduino)

*/

//#include <Arduino.h>		// by including this lib - the usage of most ARduino libraries is made possible - not necessary if you are not using any arduino instructions
#include <avr/io.h>			// This lib is needed for the C compiler to know the correct addresses for registers like DDRB, PORTB, ...
#include <util/delay.h>		// This lib is needed to calculate the correct delays for "_delay_ms(10);" 
//- the clockspeed of the ARduino Leonardo is automatically set when opening an Arduino project under STUDIO 7

char ADRESULT = 0;
int PERIOD = 0;

unsigned char READ_ADC_BYTE_CHANNEL( unsigned char channel ); // function prototype


int main (void)			// main program
{
  DDRC = 0b11000000; // pinC7 & C6 output
  DDRB = 0b11100000; // pinB7, B6, B5 output
  DDRD = 0b10000000; // pinC7 = blue led = output

  //Timer 1 PWM configuration - datasheet page 130
  // Set for Fast PWM _ 8 BIT - CLK/1024
  // OC1A = PWM at B5
  // OC1B = PWM at B6
  // OC1C = PWM at B7
  TCCR1A = 0b10101001;		//		COM1A1	COM1A0	COM1B1	COM1B0	COM1C1	COM1C0	WGM11	WGM10
  TCCR1B = 0b00001101;			//	ICNC1	ICES1   –		WGM13	WGM12	CS12	CS11	CS10

  //Timer 3 PWM configuration - datasheet page 130
  // Set for Fast PWM _ 8 BIT - CLK/1024
  // OC3A = PWM at C6

  TCCR3A = 0b10000001;		//		COM1A1	COM1A0	COM1B1	COM1B0	COM1C1	COM1C0	WGM11	WGM10
  TCCR3B = 0b00001101;			//	ICNC1	ICES1   –		WGM13	WGM12	CS12	CS11	CS10
  
  // Timer 4 PWM configuration - datasheet page 160
  // Set for Fast PWM - CLK/1024
  // OC4A = PWM at pin C7 - BLUE LED
  // OC4B = PWM at pin B6 - not used - double with OC1B
  // OC4D = PWM at pin D7
  TCCR4A = 0b10100011; //	COM4A1	COM4A0	COM4B1	COM4B0	FOC4A	FOC4B	PWM4A	PWM4B 
  TCCR4B = 0b00001011; //	PWM4X	PSR4	DTPS41	DTPS40	CS43	CS42	CS41	CS40
  TCCR4C = 0b10101001; //	COM4A1S COM4A0S COM4B1S COMAB0S COM4D1	COM4D0	FOC4D	PWM4D  (! be aware that 7:4 are shadow registers of TCCR4A and need to be programmed the same)
  TCCR4D = 0b00000000; //	FPIE4	FPEN4	FPNC4	FPES4	FPAC4	FPF4	WGM41	WGM40  (WGM = 00 = fast PWM)


  
    while(1)				// while(1) is an ever lasting loop
		{
		ADRESULT = READ_ADC_BYTE_CHANNEL(7); // convert the analog signal of channel 7 (ARduino A0) into a 8 bit digital signal and store it into "ADRESULT"
				
		OCR1AL = ADRESULT; 	//PWM value at pin B5 (OC1A)
		OCR1BL = ADRESULT; 	//PWM value at pin B6 (OC1B)
		OCR1CL = ADRESULT; 	//PWM value at pin B7 (OC1C)

		OCR3AL = ADRESULT;  //PWM value at pin C6 (OC3A)
		
		OCR4A = ADRESULT;	//PWM value at pin C7 (OC4A) - BLUE LED
		OCR4D = ADRESULT;   //PWM value at pin D7 (OC4D)
		}
}


// AD FUNCTION
// returns a 8 bit result from an AD conversion from a certain ADC channel
//Channels 0-17
// A 8 bit result needs to be loaded in a BYTE variable
unsigned char READ_ADC_BYTE_CHANNEL( unsigned char channel ){
	
	DDRF &= ~(1<<channel); // config selected channel as input

	// Left adjust + choice of channel
	ADMUX = 0b01100000 +  channel ;
	
	// Activate ADC - Stop conversion - prescaler 128 (for ADC 50K<..<200k) (16Mhz / 128 = 125000)
	ADCSRA = 0b10000111;
	
	// start conversion and wait for completion
	ADCSRA |=(1<<ADSC);
	while (ADCSRA & (1<<ADSC));
	
	// return A/D conversion result
	return ADCH;
}
